local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage")

if ("GET" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    local driver_name = args["DriverName"]
    local pk = args["ProductKey"] 
    local dn = args["DeviceName"]
    local tag = args["DeviceTags"]
    local page_size = tonumber(args["PageSize"])
    local cur_page = tonumber(args["CurrentPage"])
    local sort_method = args["SortMethod"]
    
    if (driver_name and "" == string.gsub(driver_name, '%s', "")) then
        driver_name = nil -- treat space characters as empty
    end

    if (pk and "" == string.gsub(pk, '%s', "")) then
        pk = nil -- treat space characters as empty
    end

    if (dn and "" == string.gsub(dn, '%s', "")) then
        dn = nil -- treat space characters as empty
    end

    if (sort_method == nil) or ((sort_method ~= "TimeDesc") and (sort_method ~= "TimeAsc")) then
        sort_method = "TimeDesc"
    end

    local tag_list = {}
    if (tag and "string" == type(tag)) then
        -- convert to single tag as tag list
        table.insert(tag_list, tag) 
    else
        tag_list = tag
    end

    utils.check_page_size(page_size)
    utils.check_current_page(cur_page)

    total, authed_list = utils_device.get_authed_device_list(pk, dn, driver_name, tag_list, page_size, cur_page, sort_method)
    res = {}
    res["TotalNum"] = total
    res["PageSize"] = page_size
    res["CurrentPage"] = cur_page
    res["List"] = authed_list
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
