#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ====#====#====#====
# author = "jys"
# file: ut_dimu.py
# time: 1/17/18 2:57 AM
# ====#====#====#====

import sys

sys.path.append("..")

from leda_python.mbus import MbusBase
from gi.repository import GLib
import dbus.mainloop.glib
import dbus.service
import threading
import logging
import signal
import json
import time
import dbus

DIMU_WELL_KNOWN_NAME = "iot.dmp.dimu"
DIMU_OBJECT_PATH = "/iot/dmp/dimu"
DIMU_INTERFACE = "iot.dmp.dimu"

_logger = logging.getLogger(__name__)

deviceOnlineCnt = 0
deviceOfflineCnt = 0
deviceRegisterCnt = 0


class MbusObject(dbus.service.Object):
    def __init__(self, conn, objPath):
        super(MbusObject, self).__init__(conn, objPath)

    @dbus.service.method(DIMU_INTERFACE, in_signature='s', out_signature='s')
    def registerDevice(self, inparams):
        global deviceRegisterCnt
        deviceRegisterCnt += 1
        # _logger.debug("dimu|registerDevice.inparams: %s", inparams)

        inMsgDict = json.loads(inparams)
        _logger.debug("registerDevice inparms")
        _logger.debug("\tdevice.productKey: %s", inMsgDict["productKey"])
        _logger.debug("\tdevice.deviceLocalId: %s", inMsgDict["deviceLocalId"])
        _logger.debug("\tdevice.productMD5: %s", inMsgDict["productMD5"])
        _logger.debug("\tdevice.deviceProfile: %s", inMsgDict["deviceProfile"])
        _logger.debug("\tdevice.isLocal: %s", inMsgDict["isLocal"])

        cloudId = inMsgDict["productKey"] + '_' + inMsgDict["deviceLocalId"]
        code = 0

        retJson = {
            "code": code,
            "message": "success",
            "params": {
                "deviceCloudId": cloudId
            }
        }
        _logger.debug("\tsuccess")
        return json.dumps(retJson)

    @dbus.service.method(DIMU_INTERFACE, in_signature='s', out_signature='s')
    def startupDevice(self, inparams):
        global deviceOnlineCnt
        inMsgDict = json.loads(inparams)
        code = 0
        deviceOnlineCnt += 1
        _logger.debug("the online device: %s, deviceOnlineCnt: %d", inMsgDict["deviceCloudId"], deviceOnlineCnt)

        _logger.debug("###########################")
        code = 0
        retJson = {
            "code": code,
            "message": "success"
        }
        return json.dumps(retJson)

    @dbus.service.method(DIMU_INTERFACE, in_signature='s', out_signature='s')
    def shutdownDevice(self, inparams):
        global deviceOfflineCnt
        _logger.debug("the offline device: %s", inparams)
        code = 0
        deviceOfflineCnt += 1
        retJson = {
            "code": code,
            "message": "success"
        }
        _logger.debug("the offline device: %d", deviceOfflineCnt)
        return json.dumps(retJson)


def createThread(task, args=()):
    t = threading.Thread(target=task, args=args)
    t.setDaemon(True)
    t.start()


g_Flag = False


def Task_mainLoop():
    mainloop = GLib.MainLoop()
    mainloop.run()


def Signal_handle(signum, frame):
    global g_Flag
    print('recv a signal: %s' % (frame))
    g_Flag = True


if (__name__ == "__main__"):

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    mbusConnectObject = MbusBase(DIMU_WELL_KNOWN_NAME)
    bus = mbusConnectObject.getBus()

    instance = MbusObject(bus, DIMU_OBJECT_PATH)
    createThread(Task_mainLoop)
    signal.signal(signal.SIGINT, Signal_handle)
    while (1):
        time.sleep(0.5)
        if (g_Flag): break
